@@ -43,9 +43,6 @@
 #include "gvfsudisks2drive.h"
 #include "gvfsudisks2utils.h"
 
-#define BUSY_UNMOUNT_NUM_ATTEMPTS              5
-#define BUSY_UNMOUNT_MS_DELAY_BETWEEN_ATTEMPTS 100
-
 typedef struct _GVfsUDisks2MountClass GVfsUDisks2MountClass;
 struct _GVfsUDisks2MountClass
 {
@@ -566,6 +563,29 @@ unmount_data_unref (UnmountData *data)
     }
 }
 
+static gboolean
+unmount_operation_is_eject (GMountOperation *op)
+{
+  return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (op), "x-udisks2-is-eject"));
+}
+
+static void
+unmount_data_complete (UnmountData *data,
+                       gboolean     complete_idle)
+{
+  if (data->mount_operation &&
+      !unmount_operation_is_eject (data->mount_operation))
+    gvfs_udisks2_unmount_notify_stop (data->mount_operation);
+
+  if (complete_idle)
+    g_simple_async_result_complete_in_idle (data->simple);
+  else
+    g_simple_async_result_complete (data->simple);
+
+  data->completed = TRUE;
+  unmount_data_unref (data);
+}
+
 static void unmount_do (UnmountData *data, gboolean force);
 
 static gboolean
@@ -611,9 +631,7 @@ on_mount_op_reply (GMountOperation       *mount_operation,
                                        G_IO_ERROR_FAILED_HANDLED,
                                        "GMountOperation aborted (user should never see this "
                                        "error since it is G_IO_ERROR_FAILED_HANDLED)");
-      g_simple_async_result_complete_in_idle (data->simple);
-      data->completed = TRUE;
-      unmount_data_unref (data);
+      unmount_data_complete (data, TRUE);
     }
   else if (result == G_MOUNT_OPERATION_HANDLED)
     {
@@ -629,9 +647,7 @@ on_mount_op_reply (GMountOperation       *mount_operation,
                                        G_IO_ERROR,
                                        G_IO_ERROR_BUSY,
                                        _("One or more programs are preventing the unmount operation."));
-      g_simple_async_result_complete_in_idle (data->simple);
-      data->completed = TRUE;
-      unmount_data_unref (data);
+      unmount_data_complete (data, TRUE);
     }
 }
 
@@ -693,7 +709,7 @@ lsof_command_cb (GObject       *source_object,
     {
       gboolean is_eject;
 
-      is_eject = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data->mount_operation), "x-udisks2-is-eject"));
+      is_eject = unmount_operation_is_eject (data->mount_operation);
 
       /* We want to emit the 'show-processes' signal even if launching
        * lsof(1) failed or if it didn't return any PIDs. This is because
@@ -770,9 +786,7 @@ lock_cb (GObject       *source_object,
                                           res,
                                           &error))
     g_simple_async_result_take_error (data->simple, error);
-  g_simple_async_result_complete (data->simple);
-  data->completed = TRUE;
-  unmount_data_unref (data);
+  unmount_data_complete (data, FALSE);
 }
 
 static void
@@ -813,9 +827,7 @@ unmount_cb (GObject       *source_object,
         }
     }
 
-  g_simple_async_result_complete (data->simple);
-  data->completed = TRUE;
-  unmount_data_unref (data);
+  unmount_data_complete (data, FALSE);
  out:
   ;
 }
@@ -841,18 +853,14 @@ umount_command_cb (GObject       *source_object,
                                         &error))
     {
       g_simple_async_result_take_error (data->simple, error);
-      g_simple_async_result_complete (data->simple);
-      data->completed = TRUE;
-      unmount_data_unref (data);
+      unmount_data_complete (data, FALSE);
       goto out;
     }
 
   if (WIFEXITED (exit_status) && WEXITSTATUS (exit_status) == 0)
     {
       gvfs_udisks2_volume_monitor_update (data->mount->monitor);
-      g_simple_async_result_complete (data->simple);
-      data->completed = TRUE;
-      unmount_data_unref (data);
+      unmount_data_complete (data, FALSE);
       goto out;
     }
 
@@ -867,10 +875,8 @@ umount_command_cb (GObject       *source_object,
   g_simple_async_result_set_error (data->simple,
                                    G_IO_ERROR,
                                    G_IO_ERROR_FAILED,
-                                   standard_error);
-  g_simple_async_result_complete (data->simple);
-  data->completed = TRUE;
-  unmount_data_unref (data);
+                                   "%s", standard_error);
+  unmount_data_complete (data, FALSE);
 
  out:
   g_free (standard_error);
@@ -882,6 +888,11 @@ unmount_do (UnmountData *data,
 {
   GVariantBuilder builder;
 
+  if (data->mount_operation != NULL)
+    gvfs_udisks2_unmount_notify_start (data->mount_operation, 
+                                       G_MOUNT (data->mount), NULL,
+                                       (data->filesystem == NULL));
+
   /* Use the umount(8) command if there is no block device / filesystem */
   if (data->filesystem == NULL)
     {
@@ -951,9 +962,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
   if (mount->is_burn_mount)
     {
       /* burn mounts are really never mounted so complete successfully immediately */
-      g_simple_async_result_complete_in_idle (data->simple);
-      data->completed = TRUE;
-      unmount_data_unref (data);
+      unmount_data_complete (data, TRUE);
       goto out;
     }
 
@@ -970,9 +979,8 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                            G_IO_ERROR,
                                            G_IO_ERROR_FAILED,
                                            "No object for D-Bus interface");
-          g_simple_async_result_complete (data->simple);
-          data->completed = TRUE;
-          unmount_data_unref (data);
+
+          unmount_data_complete (data, FALSE);
           goto out;
         }
       data->filesystem = udisks_object_get_filesystem (UDISKS_OBJECT (object));
@@ -987,9 +995,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                                G_IO_ERROR,
                                                G_IO_ERROR_FAILED,
                                                "No filesystem or encrypted interface on D-Bus object");
-              g_simple_async_result_complete (data->simple);
-              data->completed = TRUE;
-              unmount_data_unref (data);
+              unmount_data_complete (data, FALSE);
               goto out;
             }
 
@@ -1005,9 +1011,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                                    G_IO_ERROR,
                                                    G_IO_ERROR_FAILED,
                                                    "No filesystem interface on D-Bus object for cleartext device");
-                  g_simple_async_result_complete (data->simple);
-                  data->completed = TRUE;
-                  unmount_data_unref (data);
+                  unmount_data_complete (data, FALSE);
                   goto out;
                 }
             }
