@@ -875,7 +875,7 @@ mount_command_cb (GObject       *source_object,
   g_simple_async_result_set_error (data->simple,
                                    G_IO_ERROR,
                                    G_IO_ERROR_FAILED,
-                                   standard_error);
+                                   "%s", standard_error);
   g_simple_async_result_complete (data->simple);
   mount_data_free (data);
 
@@ -1069,6 +1069,9 @@ unlock_cb (GObject       *source_object,
             case G_PASSWORD_SAVE_PERMANENTLY:
               keyring = GNOME_KEYRING_DEFAULT;
               break;
+            default:
+              keyring = GNOME_KEYRING_DEFAULT;
+              break;
             }
 
           display_name = g_strdup_printf (_("Encryption passphrase for %s"),
